# Llama 2 UPB (gAIna)

## Minimum hardware requirements to run the model

The 7B Llama2 model demands around 16GB of vRAM for weights + KV cache (which
depends on the maximum context length). The 13B model demands around 28GB of
vRAM, and the bigger 70B model around 140GB of vRAM. Note that you need a big
enough RAM to transfer the weights from disk to vRAM.

## How to use

There are a few requirements to get a model to run. Broadly speaking, these are
the actual model (the code), the weights and the Python script to open a dialog,
as well as some Python packages.

Dockerfiles are provided to build model images from scratch using the above.
Docker images are already built (see
[here](https://gitlab.cs.pub.ro/netsys/llama-images/container_registry)), so you
can use that instead (you need to be logged in).

Other than that, an Nvidia Container Toolkit driver is necessary to run Nvidia
code on the GPU inside a docker container.

### Locally

Steps:
1. Install [Nvidia Container Toolkit (steps for
   Ubuntu)](https://saturncloud.io/blog/how-to-install-pytorch-on-the-gpu-with-docker/).
   Necessary to let docker containers use the GPU.
2. Download the Docker container image of choice (e.g. `docker image pull
   gitlab.cs.pub.ro:5050/netsys/llama-images:llama-7b-chat`).
3. Run the docker image with `docker run -it --gpus all
   gitlab.cs.pub.ro:5050/netsys/llama-images:llama-7b-chat`. This should take a
   while to load, but then a prompt to interact with Llama is displayed.

### On the UPB cluster (fep)

<strike>

```
Steps:
0. The Nvidia Container Toolkit is already installed on the cluster, so skip
   that.
1. Log in to fep (`ssh <username>@fep.grid.pub.ro`).
2. Get a bash shell into a partition with a GPU (`srun -p xl --gres
   gpu:tesla_p100:1 --mem=40G --pty bash`).
3. Pull and build the docker image into an apptainer image on the grid
   (`apptainer pull
   docker://gitlab.cs.pub.ro:5050/netsys/llama-images:llama-7b-chat`). This will
   take a while (probably around ~40 minutes). If it fails because of a timeout,
   simply run the same command again.
4. Run the apptainer image with `apptainer run --nv
   docker://gitlab.cs.pub.ro:5050/netsys/llama-images:llama-7b-chat`. The first
   time it should take about 3 minutes for it to start, but subsequent runs will
   take a few seconds (subsequent run = don't log out).
5. ???
6. Profit

*Note*: The script will sometimes still error out because of Out-of-Memory
errors or because the context length was reached. If that happens, reissue the
command to start a new dialog.
```

</strike>

Increased the context length of all images to 4096 tokens, therefore the cluster
GPUs won't be able to run the images anymore. To make it work again on the fep
cluster, you have to manually modify the context length inside a Dockerfile for
one of the LLama 7B images (to something like 512), and build the image yourself.
